local assets =
{
    Asset("ANIM", "anim/fruitbat.zip"),
    Asset("SOUND", "sound/bat.fsb"),
}

local prefabs =
{
    "batwing",
	"cherry",
}

local brain = require "brains/fruitbatbrain"

SetSharedLootTable( 'fruitbat',
{
    {'batwing',		0.15},
	{'cherry', 		0.80};
})

local SLEEP_DIST_FROMHOME = 1
local SLEEP_DIST_FROMTHREAT = 20

local function OnSleepGoHome(inst)
    inst._hometask = nil
    local home = inst.components.homeseeker ~= nil and inst.components.homeseeker.home or nil
    if home ~= nil and home:IsValid() and home.components.childspawner ~= nil then
        home.components.childspawner:GoHome(inst)
    end
end

local function OnIsDay(inst, isday)
    if isday then
        if inst._hometask == nil then
            inst._hometask = inst:DoTaskInTime(10 + math.random(), OnSleepGoHome)
        end
    elseif inst._hometask ~= nil then
        inst._hometask:Cancel()
        inst._hometask = nil
    end
end

local function StopWatchingDay(inst)
    inst:StopWatchingWorldState("isday", OnIsDay)
    if inst._hometask ~= nil then
        inst._hometask:Cancel()
        inst._hometask = nil
    end
end

local function StartWatchingDay(inst)
    inst:WatchWorldState("isday", OnIsDay)
    OnIsDay(inst, TheWorld.state.isday)
end

--	local function OnEntitySleep(inst)
--	inst:ListenForEvent("enterlimbo", StopWatchingDay)
--	inst:ListenForEvent("exitlimbo", StartWatchingDay)
--	if not inst:IsInLimbo() then
--		StartWatchingDay(inst)
--	end
--	end

--	local function OnEntityWake(inst)
--	inst:RemoveEventCallback("enterlimbo", StopWatchingDay)
--	inst:RemoveEventCallback("exitlimbo", StartWatchingDay)
--	if not inst:IsInLimbo() then
--		StopWatchingDay(inst)
--		end
--	end

local function OnPreLoad(inst, data)
	local x, y, z = inst.Transform:GetWorldPosition()
	if y > 0 then
		inst.Transform:SetPosition(x, 0, z)
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
	inst.entity:AddLight()
    inst.entity:AddNetwork()

    MakeGhostPhysics(inst, 1, .5)

    inst.DynamicShadow:SetSize(2.5, .75)

    inst.Transform:SetFourFaced()
    inst.Transform:SetScale(.75, .75, .75)

    inst.AnimState:SetBank("fruitbat")
    inst.AnimState:SetBuild("fruitbat")
	inst.AnimState:PlayAnimation("fly_loop", true)

    inst:AddTag("cavedweller")
    inst:AddTag("bat")
    inst:AddTag("flying")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("locomotor")
    inst.components.locomotor:EnableGroundSpeedMultiplier(false)
    inst.components.locomotor:SetTriggersCreep(false)
    inst.components.locomotor.walkspeed = 9

    inst:SetStateGraph("SGfruitbat")
    inst:SetBrain(brain)

    inst:AddComponent("eater")
    inst.components.eater:SetDiet({ FOODTYPE.VEGGIE }, { FOODTYPE.VEGGIE })
    inst.components.eater.strongstomach = true
	
	inst.Light:SetRadius(1)
	inst.Light:SetFalloff(0.9)
	inst.Light:SetIntensity(0.5)
	inst.Light:SetColour(255/255, 255/255, 255/255)
	inst.Light:Enable(true)

    --inst:AddComponent("sleeper")
    --inst.components.sleeper:SetResistance(1)
    --inst.components.sleeper.sleeptestfn = NocturnalSleepTest
    --inst.components.sleeper.waketestfn = NocturnalWakeTest

    inst:AddComponent("combat")
    --inst.components.combat.hiteffectsymbol = "bat_body"

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(100)

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('fruitbat')

    inst:AddComponent("inventory")

    --inst:AddComponent("periodicspawner")
    --inst.components.periodicspawner:SetPrefab("guano")
    --inst.components.periodicspawner:SetRandomTimes(120,240)
    --inst.components.periodicspawner:SetDensityInRange(30, 2)
    --inst.components.periodicspawner:SetMinimumSpacing(8)
    --inst.components.periodicspawner:Start()

    inst:AddComponent("inspectable")

    inst:AddComponent("knownlocations")

    --MakeMediumBurnableCharacter(inst, "bat_body")
    --MakeMediumFreezableCharacter(inst, "bat_body")

    MakeHauntablePanic(inst)

    --inst.OnEntitySleep = OnEntitySleep
    --inst.OnEntityWake = OnEntityWake
    inst.OnPreLoad = OnPreLoad

    return inst
end

return Prefab("fruitbat", fn, assets, prefabs)
